"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Hooks_1 = require("./Hooks");
const Router_1 = require("../Router");
const Request_1 = require("../Request");
const Response_1 = require("../Response");
const PreCompiler_1 = require("./PreCompiler");
const HttpContext_1 = require("../HttpContext");
const RequestHandler_1 = require("./RequestHandler");
const MiddlewareStore_1 = require("../MiddlewareStore");
const ExceptionManager_1 = require("../ExceptionManager");
class Server {
    constructor(_container, _logger, _profiler, _encryption, _httpConfig) {
        this._container = _container;
        this._logger = _logger;
        this._profiler = _profiler;
        this._encryption = _encryption;
        this._httpConfig = _httpConfig;
        this.middleware = new MiddlewareStore_1.MiddlewareStore(this._container);
        this.router = new Router_1.Router(this._encryption, (route) => this._precompiler.compileRoute(route));
        this.hooks = new Hooks_1.Hooks();
        this._precompiler = new PreCompiler_1.PreCompiler(this._container, this.middleware);
        this._exception = new ExceptionManager_1.ExceptionManager(this._container);
        this._requestHandler = new RequestHandler_1.RequestHandler(this.middleware, this.router);
    }
    async _handleRequest(ctx) {
        const shortcircuit = await this.hooks.executeBefore(ctx);
        if (!shortcircuit) {
            await this._requestHandler.handle(ctx);
        }
    }
    _getProfileRow(request) {
        return this._profiler.create('http:request', {
            request_id: request.id(),
            url: request.url(),
            method: request.method(),
        });
    }
    _getContext(request, response, profilerRow) {
        return new HttpContext_1.HttpContext(request, response, this._logger.child({
            request_id: request.id(),
            serializers: {},
        }), profilerRow);
    }
    errorHandler(handler) {
        this._exception.registerHandler(handler);
        return this;
    }
    optimize() {
        this.router.commit();
        this.hooks.commit();
        this._requestHandler.commit();
    }
    async handle(req, res) {
        const request = new Request_1.Request(req, res, this._encryption, this._httpConfig);
        const response = new Response_1.Response(req, res, this._httpConfig);
        const requestAction = this._getProfileRow(request);
        const ctx = this._getContext(request, response, requestAction);
        try {
            await this._handleRequest(ctx);
        }
        catch (error) {
            await this._exception.handle(error, ctx);
        }
        try {
            await this.hooks.executeAfter(ctx);
        }
        catch (error) {
            await this._exception.handle(error, ctx);
        }
        requestAction.end({ status_code: res.statusCode });
        ctx.response.finish();
    }
}
exports.Server = Server;
